import requests
import os
import json
import logging
import configparser
import boto3
import sys

from pathlib import Path
from enum import Enum
from . import ROOT_DIR

logging.basicConfig(stream=sys.stdout, level=os.environ.get('LOGLEVEL', 'INFO').upper())

logging.getLogger('boto3').setLevel(logging.CRITICAL)
logging.getLogger('botocore').setLevel(logging.CRITICAL)
logging.getLogger('s3transfer').setLevel(logging.CRITICAL)
logging.getLogger('urllib3').setLevel(logging.CRITICAL)

logger = logging.getLogger(__name__)

json_config = json.load(open(os.path.join(ROOT_DIR, "config.json"), "r"))

BASE_URL = json_config['BASE_API_URL']
CLIENT_ID = json_config['USER_POOL_CLIENT_ID']
PROVISION_URL = f"{BASE_URL}/compute/config"

class ComputeStatus():

    NO_VALUE = 0
    INIT = 1
    CREATING = 2
    CREATED = 3
    FAILED = 4

class Account():

    def __init__(self, email, password):

        self.email = email
        self.password = password

        self.user_home = str(Path.home())


    def provision(self, is_first_provision):
        data = dict(
            company="solo",
            project="default"
        )
        logger.debug("provision()")
        logger.debug(data)
        r = requests.post(PROVISION_URL,
            json=data,
            headers = {
                "x-api-key": self.auth_token
            }
        )
        logger.debug(r.status_code)
        
        if r.status_code == 202:
            return ComputeStatus.INIT
        elif r.status_code == 207:
            return ComputeStatus.CREATING
        elif r.status_code != 200:
            raise Exception(r.status_code, r.text)

        response_data = r.json()

        if is_first_provision:
            self.login()

        response_data['config']['AERO_ID_TOKEN'] = self.id_token

        # Create Metaflow directory
        if not os.path.exists(f"{self.user_home}/.metaflowconfig"):
            os.mkdir(f"{self.user_home}/.metaflowconfig")

        with open(f"{self.user_home}/.metaflowconfig/config.json", 'w', encoding='utf-8') as f:
            json.dump(response_data['config'], f, ensure_ascii=False, indent=4)

        return ComputeStatus.CREATED
        
    def login(self):

        self.auth_token, self.id_token = self._login()


    def create(self):

        outcome = self._create_account(self.email, self.password)

        return outcome
    
    def _hash(self, password):

        return sha256(password.strip()).hexdigest()

    def _create_account(cls, email, password):

        data = dict(
            username=email, 
            password=password
        )
        logger.debug("_create_account()")
        logger.debug(data)
        logger.info("Signup is currently not supported on the CLI")

        return True

    def _login(self):

        cognito = boto3.client('cognito-idp', region_name='eu-west-1')

        logger.debug("_login()")
        response = cognito.initiate_auth(
            AuthFlow="USER_PASSWORD_AUTH",
            AuthParameters={
                "USERNAME": self.email,
                "PASSWORD": self.password
            },
            ClientId=CLIENT_ID
        )
        logger.debug(response)

        if 'AuthenticationResult' not in response:
            raise Exception("Error logging in")

        logger.debug(response['AuthenticationResult']['AccessToken'])

        return response['AuthenticationResult']['AccessToken'], response['AuthenticationResult']['IdToken']



import click
import time
import os
import logging
import sys

from .version import __version__
from .account import Account, ComputeStatus

logging.basicConfig(stream=sys.stdout, level=os.environ.get('LOGLEVEL', 'INFO').upper())

logging.getLogger('boto3').setLevel(logging.CRITICAL)
logging.getLogger('botocore').setLevel(logging.CRITICAL)
logging.getLogger('s3transfer').setLevel(logging.CRITICAL)
logging.getLogger('urllib3').setLevel(logging.CRITICAL)

logger = logging.getLogger(__name__)

site_base = "https://site.aeroplatform.co.uk"
name_lower = "aero"
name_upper = "Aero"

def echo(line, **kwargs):
    click.secho(line, **kwargs)

def cyan(string):
    return click.style(string, fg='cyan')

def magenta(string):
    return click.style(string, fg='magenta')

def red(string):
    return click.style(string, fg='red')

@click.group()
def cli():
    echo(f'{magenta(name_upper + " Platform")} {magenta("v" + __version__)}')
    echo("A Data Platform designed to put developers first.")
    echo(f"For more information, visit {cyan(site_base)}")
    echo("")
    pass

@cli.group(help="Account management")
def account():
    pass

@account.command(help="Login to an account")
@click.option("--email", default=None,
    help="Email address")
@click.option("--password", default=None,
    help="Password")
def login(password, email):

    if not email and not password:
        email = click.prompt("Email > ")
        password = click.prompt("Password > ", hide_input=True)

    try:
        account = Account(email, password)
        _login(account)
    except Exception as e:
        raise e
        echo("An error occured when attempting to log in, please try again")
        logger.debug(e)
        return

    try:
        _provision(account)

    except Exception as e:
        echo("An error occured when attempting to provision, please try again")
        logger.debug(e)

    return

@account.command(help="Create an account")
def create():

    echo(f'Welcome to {magenta(name_upper)}!')
    echo(f'Please create an account from our website, {site_base}.')

    return


def _login(account):

    try:
        account.login()
    except Exception as e:

        if str(e) == "403":
            echo(f'{red("Error: ")} Incorrect email/password')
            raise e
        elif str(e) == "404":
            echo(f'{red("Error: ")} User doesnt exist')
            raise e
        else:
            echo("An error has occurred")
            raise e
    
    echo(f'{cyan("Login Successful")}')

def _provision(account):

    status = ComputeStatus.NO_VALUE
    i = 0
    is_first_provision = False

    while status != ComputeStatus.CREATED:
        status = account.provision(is_first_provision)
        logger.debug(f"Current Status is {status}, iteration {i}")

        if status == ComputeStatus.INIT:
            echo(f"Initialised Compute Environment Creation")
            is_first_provision = True

        if status == ComputeStatus.CREATING:
            echo(f"Creating Compute Environment...")   
            is_first_provision = True

        if status == ComputeStatus.CREATED:
            echo(f"Compute Environment Created")
            echo(f"You are now ready to run Flows with Aero. For some basic tutorials, visit: {site_base}/commands")
            break  

        time.sleep(30)
        i += 1
